% Simulate an indirect MRAC
% for a motor k/(1+sT)

% True system
noiselevel = 0.005;
k = 1;   
T = 5; 
s=tf('s');
G=k/(s*(1+s*T));
h = 1;     % sample period
H=c2d(G,h);
[btrue,atrue] = tfdata(H,'v');
btrue=btrue(2:end);
d0=1;
b0=btrue(1);

% Reference Signal
N = 400;   % nr of samples
time=(1:N)';
wc = 0.05; % square wave reference signal
uc = 1*sign(sin(wc*time));
%uc = randn(size(time)); % better excitation

%Ideal controller
am=poly([0.8]);
ao=[1 -0.6];
[r0,s0,t0]=rstd(btrue,1,atrue,1,am,ao);
nrpar = length(r0) + length(s0);

% initialise vectors
u=zeros(N,1);
y=zeros(N,1);
uf=zeros(N,1);
yf=zeros(N,1);
theta=zeros(nrpar,N);
theta(:,1)=[1;0;0;0];
theta(:,2)=[1;0;0;0];

phi = zeros(nrpar,N);
K=zeros(nrpar,N);
e=zeros(N,1);
Pinv=zeros(nrpar,nrpar,N);
Pinv(:,:,2)=1e-2*eye(nrpar);

amao=conv(am,ao);
for k=3:N  
  n = noiselevel*randn;
  y(k) = -atrue(2:end)*y(k-1:-1:k-2) + btrue*u(k-1:-1:k-2)+n;
  yf(k) = -amao(2:end)*yf(k-1:-1:k-2) + y(k);
  
  phi(:,k) = [uf(k-d0); uf(k-d0-1); yf(k-d0); yf(k-d0-1)];
  Pinv(:,:,k) = Pinv(:,:,k-1) + phi(:,k)*phi(:,k)';
  K(:,k) = Pinv(:,:,k)\phi(:,k);
  e(k) = y(k)-phi(:,k)'*theta(:,k-1);
  theta(:,k) = theta(:,k-1) + K(:,k)*e(k); 
  
  r=[theta(1,k) theta(2,k)]; 
  s=[theta(3,k) theta(4,k)];
  t = ao*sum(am);
  u(k) = 1/r(1)*(-r(2)*u(k-1) - s*y(k:-1:k-1) + t*uc(k:-1:k-1)); 
  uf(k) = -amao(2:end)*uf(k-1:-1:k-2) + u(k);
end

figure(1)
title('parameter estimates')
subplot(311)
plot(theta(2,:)./theta(1,:));
title('r1/r0')
hold on;
plot([1 N],[r0(2) r0(2)],'r')
subplot(312)
plot(theta(3,:)./theta(1,:));
title('s0/r0')
hold on;
plot([1 N],[s0(1) s0(1)],'r')
subplot(313)
plot(theta(4,:)./theta(1,:));
title('s1/r0')
hold on;
plot([1 N],[s0(2) s0(2)],'r')

figure(2)
subplot(211)
title('adaptive control of motor')
plot(y);grid on; hold on
subplot(212)
plot(u);grid on; hold on

